@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright © 2010-2011 Jonathan Thomas <echidnaman@kubuntu.org>        *
+ *   Copyright © 2010-2012 Jonathan Thomas <echidnaman@kubuntu.org>        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or         *
  *   modify it under the terms of the GNU General Public License as        *
@@ -82,6 +82,8 @@ public:
     QSet<Group> groups;
     // Cache of origin/human-readable name pairings
     QHash<QString, QString> originMap;
+    // Relation of an origin and its hostname
+    QHash<QString, QString> siteMap;
     WorkerEvent state;
 
     // Counts
@@ -114,8 +116,10 @@ public:
     bool compressEvents;
     pkgDepCache::ActionGroup *actionGroup;
 
+    // Other
     bool writeSelectionFile(const QString &file, const QString &path) const;
     void setWorkerLocale();
+    QString customProxy;
     void setWorkerProxy();
 };
 
@@ -139,7 +143,8 @@ void BackendPrivate::setWorkerLocale()
 
 void BackendPrivate::setWorkerProxy()
 {
-    worker->setProxy(QLatin1String(qgetenv("http_proxy")));
+    QString proxy = customProxy.isEmpty() ? qgetenv("http_proxy") : proxy = customProxy;
+    worker->setProxy(proxy);
 }
 
 Backend::Backend()
@@ -205,6 +210,7 @@ void Backend::reloadCache()
     d->packages.clear();
     d->groups.clear();
     d->originMap.clear();
+    d->siteMap.clear();
     d->packagesIndex.clear();
     d->installedCount = 0;
 
@@ -215,11 +221,8 @@ void Backend::reloadCache()
 
     // Populate internal package cache
     int count = 0;
-    QSet<Group> groupSet;
 
     d->isMultiArch = architectures().size() > 1;
-    bool pkgMultiArch;
-    QString arch;
 
     pkgCache::PkgIterator iter;
     for (iter = depCache->PkgBegin(); !iter.end(); ++iter) {
@@ -248,24 +251,23 @@ void Backend::reloadCache()
 
         if(!Ver.end()) {
             pkgCache::VerFileIterator VF = Ver.FileList();
-            d->originMap[QLatin1String(VF.File().Origin())] = QLatin1String(VF.File().Label());
+            QLatin1String origin(QLatin1String(VF.File().Origin()));
+            d->originMap[origin] = VF.File().Label();
+            d->siteMap[VF.File().Site()] = origin;
         }
     }
 
-    if (d->originMap.contains(QLatin1String(""))) {
-        d->originMap.remove(QLatin1String(""));
-    }
+    d->originMap.remove(QString());
 
     d->undoStack.clear();
     d->redoStack.clear();
 
     // Determine which packages are pinned for display purposes
-    QString dir = QString::fromStdString(_config->FindDir("Dir::Etc"))
-                  % QLatin1String("preferences.d/");
+    QString dirBase = d->config->findDirectory(QLatin1String("Dir::Etc"));
+    QString dir = dirBase % QLatin1String("preferences.d/");
     QDir logDirectory(dir);
     QStringList pinFiles = logDirectory.entryList(QDir::Files, QDir::Name);
-    pinFiles << QString::fromStdString(_config->FindDir("Dir::Etc")) %
-                QLatin1String("preferences");
+    pinFiles << dirBase % QLatin1String("preferences");
 
     Q_FOREACH (const QString &pinName, pinFiles) {
         QString pinPath;
@@ -324,12 +326,19 @@ Cache *Backend::cache() const
     return d->cache;
 }
 
+pkgRecords *Backend::records() const
+{
+    Q_D(const Backend);
+
+    return d->records;
+}
+
 Package *Backend::package(pkgCache::PkgIterator &iter) const
 {
     Q_D(const Backend);
 
     int index = d->packagesIndex.at(iter->ID);
-    if (index != -1 && index < d->packages.size()) {
+    if (index != -1) {
         return d->packages.at(index);
     }
     return 0;
@@ -337,7 +346,7 @@ Package *Backend::package(pkgCache::PkgIterator &iter) const
 
 Package *Backend::package(const QString &name) const
 {
-    return package(QLatin1String(name.toStdString().c_str()));
+    return package(QLatin1String(name.toLatin1()));
 }
 
 Package *Backend::package(const QLatin1String &name) const
@@ -367,6 +376,13 @@ Package *Backend::packageForFile(const QString &file) const
     return 0;
 }
 
+QStringList Backend::origins() const
+{
+    Q_D(const Backend);
+
+    return d->originMap.keys();
+}
+
 QStringList Backend::originLabels() const
 {
     Q_D(const Backend);
@@ -619,16 +635,9 @@ bool Backend::isMultiArchEnabled() const
 
 QStringList Backend::architectures() const
 {
-    std::vector<std::string> archs = APT::Configuration::getArchitectures(false);
+    Q_D(const Backend);
 
-    QStringList archList;
-    for (std::string &arch : archs)
-    {
-         archList.append(QString::fromStdString(arch));
-    }
-
-
-    return archList;
+    return d->config->architectures();
 }
 
 QString Backend::nativeArchitecture() const
@@ -1275,8 +1284,7 @@ bool Backend::setPackagePinned(Package *package, bool pin)
 {
     Q_D(Backend);
 
-    QString dir = QString::fromStdString(_config->FindDir("Dir::Etc"))
-                  % QLatin1String("preferences.d/");
+    QString dir = d->config->findDirectory("Dir::Etc") % QLatin1String("preferences.d/");
     QString path = dir % package->latin1Name();
     QString pinDocument;
 
@@ -1413,6 +1421,13 @@ bool Backend::addArchiveToCache(const DebFile &archive)
     return d->worker->copyArchiveToCache(archive.filePath());
 }
 
+void Backend::setWorkerProxy(const QString &proxy)
+{
+    Q_D(Backend);
+
+    d->customProxy = proxy;
+}
+
 void Backend::workerStarted()
 {
     Q_D(Backend);
@@ -1515,4 +1530,10 @@ void Backend::serviceOwnerChanged(const QString &name, const QString &oldOwner,
     }
 }
 
+QString Backend::originForSite(const QString& host) const
+{
+    Q_D(const Backend);
+    return d->siteMap[host];
+}
+
 }
