@@ -31,6 +31,7 @@
 #include <apt-pkg/algorithms.h>
 #include <apt-pkg/acquire-item.h>
 #include <apt-pkg/configuration.h>
+#include <apt-pkg/aptconfiguration.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/fileutl.h>
 #include <apt-pkg/init.h>
@@ -252,6 +253,7 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
 
     m_timeout->stop();
 
+    pkgDepCache::ActionGroup *actionGroup = new pkgDepCache::ActionGroup(*m_cache->depCache());
     // Parse out the argument list and mark packages for operations
     QVariantMap::const_iterator mapIter = instructionsList.constBegin();
 
@@ -279,6 +281,7 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
             emit errorOccurred(QApt::NotFoundError, args);
             emit workerFinished(false);
             m_timeout->start();
+            delete actionGroup;
             return;
         }
 
@@ -350,6 +353,8 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
         qDebug() << "fixed errors";
     }
 
+    delete actionGroup;
+
     emit workerStarted();
 
     if (_error->PendingError()) {
@@ -483,25 +488,10 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
     if (fetcher.Run() != pkgAcquire::Continue) {
         // Our fetcher will report warnings for itself, but if it fails entirely
         // we have to send the error and finished signals
-        emit errorOccurred(QApt::FetchError, QVariantMap());
-        emit workerFinished(false);
-        m_timeout->start();
-        return;
-    }
+        if (!m_acquireStatus->wasCancelled()) {
+            emit errorOccurred(QApt::FetchError, QVariantMap());
+        }
 
-    bool failed = false;
-    for (pkgAcquire::ItemIterator i = fetcher.ItemsBegin(); i != fetcher.ItemsEnd(); ++i) {
-        if((*i)->Status == pkgAcquire::Item::StatDone && (*i)->Complete)
-            continue;
-        if((*i)->Status == pkgAcquire::Item::StatIdle)
-            continue;
-
-        failed = true;
-        break;
-    }
-
-    if (failed && !packageManager->FixMissing()) {
-        emit errorOccurred(QApt::FetchError, QVariantMap());
         emit workerFinished(false);
         m_timeout->start();
         return;
@@ -618,20 +608,31 @@ void QAptWorker::installDebFile(const QString &fileName)
 
     QApt::DebFile deb(fileName);
 
-    QString arch = deb.architecture();
+    QString debArch = deb.architecture();
 
-    if (arch != QLatin1String("all") &&
-        arch != QString::fromStdString(_config->Find("APT::Architecture", ""))) {
-        // TODO report what arch was provided vs needed
-        qDebug() << arch << QString::fromStdString(_config->Find("APT::Architecture", ""));
-        emit errorOccurred(QApt::WrongArchError, QVariantMap());
+    QStringList archList;
+    archList.append(QLatin1String("all"));
+    std::vector<std::string> archs = APT::Configuration::getArchitectures(false);
+
+    for (std::string &arch : archs)
+    {
+         archList.append(QString::fromStdString(arch));
+    }
+
+    if (!archList.contains(debArch)) {
+        QVariantMap details;
+        details["RequestedArch"] = debArch;
+
+        emit errorOccurred(QApt::WrongArchError, details);
         emit workerFinished(false);
+        m_timeout->start();
         return;
     }
 
     if (!QApt::Auth::authorize(QLatin1String("org.kubuntu.qaptworker.commitChanges"), message().service())) {
         emit errorOccurred(QApt::AuthError, QVariantMap());
         emit workerFinished(false);
+        m_timeout->start();
         return;
     }
 
@@ -646,7 +647,6 @@ void QAptWorker::installDebFile(const QString &fileName)
     connect(m_dpkgProcess, SIGNAL(readyRead()), this, SLOT(updateDpkgProgress()));
     connect(m_dpkgProcess, SIGNAL(finished(int,QProcess::ExitStatus)),
             this, SLOT(dpkgFinished(int,QProcess::ExitStatus)));
-
 }
 
 void QAptWorker::dpkgStarted()
