@@ -49,10 +49,10 @@ QAptBatch::QAptBatch(QString mode, QStringList packages, int winId)
     m_worker = new OrgKubuntuQaptworkerInterface("org.kubuntu.qaptworker",
                                                   "/", QDBusConnection::systemBus(),
                                                   this);
-    connect(m_worker, SIGNAL(errorOccurred(int, const QVariantMap&)),
-            this, SLOT(errorOccurred(int, const QVariantMap&)));
-    connect(m_worker, SIGNAL(warningOccurred(int, const QVariantMap&)),
-            this, SLOT(warningOccurred(int, const QVariantMap&)));
+    connect(m_worker, SIGNAL(errorOccurred(int,QVariantMap)),
+            this, SLOT(errorOccurred(int,QVariantMap)));
+    connect(m_worker, SIGNAL(warningOccurred(int,QVariantMap)),
+            this, SLOT(warningOccurred(int,QVariantMap)));
     connect(m_worker, SIGNAL(workerStarted()), this, SLOT(workerStarted()));
     connect(m_worker, SIGNAL(workerEvent(int)), this, SLOT(workerEvent(int)));
     connect(m_worker, SIGNAL(workerFinished(bool)), this, SLOT(workerFinished(bool)));
@@ -118,14 +118,14 @@ void QAptBatch::workerStarted()
     // Reset the progresbar's maximum to default
     progressBar()->setMaximum(100);
     connect(m_watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-            this, SLOT(serviceOwnerChanged(QString, QString, QString)));
+            this, SLOT(serviceOwnerChanged(QString,QString,QString)));
 
-    connect(m_worker, SIGNAL(questionOccurred(int, const QVariantMap&)),
-            this, SLOT(questionOccurred(int, const QVariantMap&)));
-    connect(m_worker, SIGNAL(downloadProgress(int, int, int)),
-            this, SLOT(updateDownloadProgress(int, int, int)));
-    connect(m_worker, SIGNAL(commitProgress(const QString&, int)),
-            this, SLOT(updateCommitProgress(const QString&, int)));
+    connect(m_worker, SIGNAL(questionOccurred(int,QVariantMap)),
+            this, SLOT(questionOccurred(int,QVariantMap)));
+    connect(m_worker, SIGNAL(downloadProgress(int,int,int)),
+            this, SLOT(updateDownloadProgress(int,int,int)));
+    connect(m_worker, SIGNAL(commitProgress(QString,int)),
+            this, SLOT(updateCommitProgress(QString,int)));
 }
 void QAptBatch::errorOccurred(int code, const QVariantMap &args)
 {
@@ -262,7 +262,7 @@ void QAptBatch::questionOccurred(int code, const QVariantMap &args)
             KMessageBox::information(this, text, title);
             response["MediaChanged"] = true;
             m_worker->answerWorkerQuestion(response);
-        }
+        }   break;
         case QApt::InstallUntrusted: {
             QStringList untrustedItems = args["UntrustedItems"].toStringList();
 
@@ -277,19 +277,9 @@ void QAptBatch::questionOccurred(int code, const QVariantMap &args)
                         "security risk, as the presence of unverifiable software "
                         "can be a sign of tampering.</warning> Do you wish to continue?",
                         untrustedItems.size());
-            int result = KMessageBox::Cancel;
-            bool installUntrusted = false;
-
-            result = KMessageBox::warningContinueCancelList(this, text,
+            int result = KMessageBox::warningContinueCancelList(this, text,
                                                             untrustedItems, title);
-            switch (result) {
-                case KMessageBox::Continue:
-                    installUntrusted = true;
-                    break;
-                case KMessageBox::Cancel:
-                    installUntrusted = false;
-                    break;
-            }
+            bool installUntrusted = result==KMessageBox::Continue;
 
             response["InstallUntrusted"] = installUntrusted;
             m_worker->answerWorkerQuestion(response);
@@ -321,14 +311,14 @@ void QAptBatch::workerEvent(int code)
             break;
         case QApt::CacheUpdateFinished:
             setWindowTitle(i18nc("@title:window", "Refresh Complete"));
-            if (m_warningStack.size() > 0) {
+            if (!m_warningStack.isEmpty()) {
                 showQueuedWarnings();
             }
-            if (m_errorStack.size() > 0) {
+            if (!m_errorStack.isEmpty()) {
                 showQueuedErrors();
             }
 
-            if (m_errorStack.size() > 0) {
+            if (!m_errorStack.isEmpty()) {
                 setLabelText(i18nc("@info:status", "Refresh completed with errors"));
             } else {
                 setLabelText(i18nc("@label", "Package information successfully refreshed"));
@@ -362,16 +352,16 @@ void QAptBatch::workerEvent(int code)
             show(); // In case no download was necessary
             break;
         case QApt::CommitChangesFinished:
-            if (m_warningStack.size() > 0) {
+            if (!m_warningStack.isEmpty()) {
                 showQueuedWarnings();
             }
-            if (m_errorStack.size() > 0) {
+            if (!m_errorStack.isEmpty()) {
                 showQueuedErrors();
             }
             if (m_mode == "install") {
                 setWindowTitle(i18nc("@title:window", "Installation Complete"));
 
-                if (m_errorStack.size() > 0) {
+                if (!m_errorStack.isEmpty()) {
                     setLabelText(i18nc("@label",
                                        "Package installation finished with errors."));
                 } else {
@@ -382,7 +372,7 @@ void QAptBatch::workerEvent(int code)
             } else if (m_mode == "uninstall") {
                 setWindowTitle(i18nc("@title:window", "Removal Complete"));
 
-                if (m_errorStack.size() > 0) {
+                if (!m_errorStack.isEmpty()) {
                     setLabelText(i18nc("@label",
                                        "Package removal finished with errors."));
                 } else {
@@ -403,12 +393,12 @@ void QAptBatch::workerFinished(bool success)
 {
     Q_UNUSED(success);
     disconnect(m_watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-               this, SLOT(serviceOwnerChanged(QString, QString, QString)));
+               this, SLOT(serviceOwnerChanged(QString,QString,QString)));
 
-    disconnect(m_worker, SIGNAL(downloadProgress(int, int, int)),
-               this, SLOT(updateDownloadProgress(int, int, int)));
-    disconnect(m_worker, SIGNAL(commitProgress(const QString&, int)),
-               this, SLOT(updateCommitProgress(const QString&, int)));
+    disconnect(m_worker, SIGNAL(downloadProgress(int,int,int)),
+               this, SLOT(updateDownloadProgress(int,int,int)));
+    disconnect(m_worker, SIGNAL(commitProgress(QString,int)),
+               this, SLOT(updateCommitProgress(QString,int)));
 }
 
 void QAptBatch::serviceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner)
